/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"

bool sandboxGameInit(sandboxgame_t *game) {
  uint8_t i;

  // Initialize the scene.
  sceneInit(&game->scene, &game->engine);
  
  // Load Shader
  assetManagerLoadShader(&game->engine.assetManager, game->scene.assetOwner,
    &game->shader, "shaders/textured.vert", "shaders/textured.frag"
  );
  assetManagerLoadShader(&game->engine.assetManager, game->scene.assetOwner,
    &game->shader2, "shaders/test.vert", "shaders/test.frag"
  );

  // Load the texture
  assetManagerLoadScaledTexture(
    &game->engine.assetManager, game->scene.assetOwner,
    &game->st, "poker/characters/sammy", "sprite"
  );
  assetManagerLoadTextureScale( 
    &game->engine.assetManager, game->scene.assetOwner,
    &game->st, &game->texture, 0
  );

  assetManagerLoadTexture(
    &game->engine.assetManager, game->scene.assetOwner,
    &game->texture, "textures/test_texture.png"
  );

  cubeInit(&game->quad, 1, 1, 1);

  i = renderPassAdd(&game->scene.renderList, &game->shader);
  i = renderPassAdd(&game->scene.renderList, &game->shader);

  return true;
}

void sandboxGameUpdate(sandboxgame_t *game) {
  camera_t camera;
  float n;

  n = assetManagerProgressGet(&game->engine.assetManager);
  if(n < 1.0f) return;

  sceneRenderStart(&game->scene);

  // First pass
  shaderuniform_t uniView = shaderGetUniform(&game->shader, "u_View");
  shaderuniform_t uniProj = shaderGetUniform(&game->shader, "u_Proj");
  shaderuniform_t uniModel = shaderGetUniform(&game->shader, "u_Modl");
  shaderuniform_t uniColor = shaderGetUniform(&game->shader, "u_Colr");
  shaderuniform_t uniTexture = shaderGetUniform(&game->shader, "u_Text");

  renderpass_t *pass =renderListRenderPass(
    &game->scene.renderList, &game->engine, 0
  );
  cameraPerspective(&camera, 45.0f,
    game->engine.render.width/game->engine.render.height, 0.001f, 1000.0f
  );
  cameraLookAt(&camera, 3,3,3, 0,0,0);
  
  shaderUseCamera(pass->shader, uniView, uniProj, &camera);
  shaderUseColor(pass->shader, uniColor, PIXEL_COLOR_WHITE);
  shaderUseTexture(pass->shader, uniTexture, &game->texture);
  shaderUsePosition(pass->shader, uniModel, 0,0,0,
    game->engine.time.current/2.0f, game->engine.time.current, 0
  );
  primitiveDraw(&game->quad, 0, -1);


  pass = renderListRenderPass(&game->scene.renderList, &game->engine, 1);
  shaderUsePosition(pass->shader, uniModel, 0,0,0,
    -game->engine.time.current/2.0f, -game->engine.time.current, 0
  );
  primitiveDraw(&game->quad, 0, -1);

  shaderuniform_t uniTextureSet[2];
  uniTextureSet[0] = shaderGetUniform(&game->shader2, "u_Text0");
  uniTextureSet[1] = shaderGetUniform(&game->shader2, "u_Text1");

  sceneRenderEnd(&game->scene, &game->shader2,
    shaderGetUniform(&game->shader2, "u_View"),
    shaderGetUniform(&game->shader2, "u_Proj"),
    shaderGetUniform(&game->shader2, "u_Modl"),
    uniTextureSet,
    &game->shader, uniView, uniProj, uniModel, uniTexture
  );
}

void sandboxGameDispose(sandboxgame_t *game) {
  sceneDispose(&game->scene);
  // assetManagerHolderRelease(&game->engine.assetManager, game->assetOwner);
  // primitiveDispose(&game->quad);
}